#!/usr/bin/env python
# Script that can read from a serial device and show timestamps.
#
# Copyright (C) 2009  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

# Usage:
#   tools/readserial.py /dev/ttyUSB0 115200

import sys
import time
import select
import serial

# Reset time counter after this much idle time.
RESTARTINTERVAL = 60
# Alter timing reports based on how much time would be spent writing
# to serial.
ADJUSTBAUD = 1
# Number of bits in a transmitted byte - 8N1 is 1 start bit + 8 data
# bits + 1 stop bit.
BITSPERBYTE = 10

def readserial(infile, logfile, baudrate):
    lasttime = 0
    byteadjust = 0.0
    if ADJUSTBAUD:
        byteadjust = float(BITSPERBYTE) / baudrate
    while 1:
        # Read data
        try:
            res = select.select([infile, sys.stdin], [], [])
        except KeyboardInterrupt:
            sys.stdout.write("\n")
            break
        if sys.stdin in res[0]:
            # Got keyboard input - force reset on next serial input
            sys.stdin.read(1)
            lasttime = 0
            if len(res[0]) == 1:
                continue
        d = infile.read(4096)
        datatime = time.time()

        datatime -= len(d) * byteadjust

        # Reset start time if no data for some time
        if datatime - lasttime > RESTARTINTERVAL:
            starttime = datatime
            charcount = 0
            isnewline = 1
            msg = "\n\n======= %s (adjust=%d)\n" % (
                time.asctime(time.localtime(datatime)), ADJUSTBAUD)
            sys.stdout.write(msg)
            logfile.write(msg)
        lasttime = datatime

        # Translate unprintable chars; add timestamps
        out = ""
        for c in d:
            if isnewline:
                delta = datatime - starttime - (charcount * byteadjust)
                out += "%06.3f: " % delta
                isnewline = 0
            oc = ord(c)
            charcount += 1
            datatime += byteadjust
            if oc == 0x0d:
                continue
            if oc == 0x00:
                out += "<00>\n"
                isnewline = 1
                continue
            if oc == 0x0a:
                out += "\n"
                isnewline = 1
                continue
            if oc < 0x20 or oc >= 0x7f and oc != 0x09:
                out += "<%02x>" % oc
                continue
            out += c

        sys.stdout.write(out)
        sys.stdout.flush()
        logfile.write(out)
        logfile.flush()

def printUsage():
    print "Usage:\n   %s [<serialdevice> [<baud>]]" % (sys.argv[0],)
    sys.exit(1)

def main():
    serialport = 0
    baud = 115200
    if len(sys.argv) > 3:
        printUsage()
    if len(sys.argv) > 1:
        serialport = sys.argv[1]
    if len(sys.argv) > 2:
        baud = int(sys.argv[2])

    ser = serial.Serial(serialport, baud, timeout=0)

    logname = time.strftime("seriallog-%Y%m%d_%H%M%S.log")
    f = open(logname, 'wb')
    readserial(ser, f, baud)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Script to analyze code and arrange ld sections.
#
# Copyright (C) 2008  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

import sys

# LD script headers/trailers
COMMONHEADER = """
/* DO NOT EDIT!  This is an autogenerated file.  See tools/layoutrom.py. */
OUTPUT_FORMAT("elf32-i386")
OUTPUT_ARCH("i386")
SECTIONS
{
"""
COMMONTRAILER = """

        /* Discard regular data sections to force a link error if
         * code attempts to access data not marked with VAR16 (or other
         * appropriate macro)
         */
        /DISCARD/ : {
                *(.text*) *(.data*) *(.bss*) *(.rodata*)
                *(COMMON) *(.discard*) *(.eh_frame)
                }
}
"""


######################################################################
# Determine section locations
######################################################################

# Align 'pos' to 'alignbytes' offset
def alignpos(pos, alignbytes):
    mask = alignbytes - 1
    return (pos + mask) & ~mask

# Determine the final addresses for a list of sections that end at an
# address.
def getSectionsStart(sections, endaddr, minalign=1):
    totspace = 0
    for size, align, name in sections:
        if align > minalign:
            minalign = align
        totspace = alignpos(totspace, align) + size
    startaddr = (endaddr - totspace) / minalign * minalign
    curaddr = startaddr
    # out = [(addr, sectioninfo), ...]
    out = []
    for sectioninfo in sections:
        size, align, name = sectioninfo
        curaddr = alignpos(curaddr, align)
        out.append((curaddr, sectioninfo))
        curaddr += size
    return out, startaddr

# Return the subset of sections with a given name prefix
def getSectionsPrefix(sections, prefix):
    lp = len(prefix)
    out = []
    for size, align, name in sections:
        if name[:lp] == prefix:
            out.append((size, align, name))
    return out

# The 16bit code can't exceed 64K of space.
BUILD_BIOS_ADDR = 0xf0000
BUILD_BIOS_SIZE = 0x10000

# Layout the 16bit code.  This ensures sections with fixed offset
# requirements are placed in the correct location.  It also places the
# 16bit code as high as possible in the f-segment.
def fitSections(sections, fillsections):
    canrelocate = list(fillsections)
    # fixedsections = [(addr, sectioninfo), ...]
    fixedsections = []
    for sectioninfo in sections:
        size, align, name = sectioninfo
        if name[:11] == '.fixedaddr.':
            addr = int(name[11:], 16)
            fixedsections.append((addr, sectioninfo))
            if align != 1:
                print "Error: Fixed section %s has non-zero alignment (%d)" % (
                    name, align)
                sys.exit(1)

    # Find freespace in fixed address area
    fixedsections.sort()
    # fixedAddr = [(freespace, sectioninfo), ...]
    fixedAddr = []
    for i in range(len(fixedsections)):
        fixedsectioninfo = fixedsections[i]
        addr, section = fixedsectioninfo
        if i == len(fixedsections) - 1:
            nextaddr = BUILD_BIOS_SIZE
        else:
            nextaddr = fixedsections[i+1][0]
        avail = nextaddr - addr - section[0]
        fixedAddr.append((avail, fixedsectioninfo))

    # Attempt to fit other sections into fixed area
    extrasections = []
    fixedAddr.sort()
    canrelocate.sort()
    totalused = 0
    for freespace, fixedsectioninfo in fixedAddr:
        fixedaddr, fixedsection = fixedsectioninfo
        addpos = fixedaddr + fixedsection[0]
        totalused += fixedsection[0]
        nextfixedaddr = addpos + freespace
#        print "Filling section %x uses %d, next=%x, available=%d" % (
#            fixedaddr, fixedsection[0], nextfixedaddr, freespace)
        while 1:
            canfit = None
            for fitsection in canrelocate:
                fitsize, fitalign, fitname = fitsection
                if addpos + fitsize > nextfixedaddr:
                    # Can't fit and nothing else will fit.
                    break
                fitnextaddr = alignpos(addpos, fitalign) + fitsize
#                print "Test %s - %x vs %x" % (
#                    fitname, fitnextaddr, nextfixedaddr)
                if fitnextaddr > nextfixedaddr:
                    # This item can't fit.
                    continue
                canfit = (fitnextaddr, fitsection)
            if canfit is None:
                break
            # Found a section that can fit.
            fitnextaddr, fitsection = canfit
            canrelocate.remove(fitsection)
            extrasections.append((addpos, fitsection))
            addpos = fitnextaddr
            totalused += fitsection[0]
#            print "    Adding %s (size %d align %d) pos=%x avail=%d" % (
#                fitsection[2], fitsection[0], fitsection[1]
#                , fitnextaddr, nextfixedaddr - fitnextaddr)
    firstfixed = fixedsections[0][0]

    # Report stats
    total = BUILD_BIOS_SIZE-firstfixed
    slack = total - totalused
    print ("Fixed space: 0x%x-0x%x  total: %d  slack: %d"
           "  Percent slack: %.1f%%" % (
            firstfixed, BUILD_BIOS_SIZE, total, slack,
            (float(slack) / total) * 100.0))

    return fixedsections + extrasections, firstfixed

def doLayout(sections16, sections32seg, sections32flat):
    # Determine 16bit positions
    textsections = getSectionsPrefix(sections16, '.text.')
    rodatasections = (getSectionsPrefix(sections16, '.rodata.str1.1')
                      + getSectionsPrefix(sections16, '.rodata.__func__.'))
    datasections = getSectionsPrefix(sections16, '.data16.')
    fixedsections = getSectionsPrefix(sections16, '.fixedaddr.')

    locs16fixed, firstfixed = fitSections(fixedsections, textsections)
    prunesections = [i[1] for i in locs16fixed]
    remsections = [i for i in textsections+rodatasections+datasections
                   if i not in prunesections]
    locs16, code16_start = getSectionsStart(remsections, firstfixed)
    locs16 = locs16 + locs16fixed
    locs16.sort()

    # Determine 32seg positions
    textsections = getSectionsPrefix(sections32seg, '.text.')
    rodatasections = (getSectionsPrefix(sections32seg, '.rodata.str1.1')
                      + getSectionsPrefix(sections32seg, '.rodata.__func__.'))
    datasections = getSectionsPrefix(sections32seg, '.data32seg.')

    locs32seg, code32seg_start = getSectionsStart(
        textsections + rodatasections + datasections, code16_start)

    # Determine 32flat positions
    textsections = getSectionsPrefix(sections32flat, '.text.')
    rodatasections = getSectionsPrefix(sections32flat, '.rodata')
    datasections = getSectionsPrefix(sections32flat, '.data.')
    bsssections = getSectionsPrefix(sections32flat, '.bss.')

    locs32flat, code32flat_start = getSectionsStart(
        textsections + rodatasections + datasections + bsssections
        , code32seg_start + BUILD_BIOS_ADDR, 16)

    # Print statistics
    size16 = BUILD_BIOS_SIZE - code16_start
    size32seg = code16_start - code32seg_start
    size32flat = code32seg_start + BUILD_BIOS_ADDR - code32flat_start
    print "16bit size:           %d" % size16
    print "32bit segmented size: %d" % size32seg
    print "32bit flat size:      %d" % size32flat

    return locs16, locs32seg, locs32flat


######################################################################
# Linker script output
######################################################################

# Write LD script includes for the given cross references
def outXRefs(xrefs, finallocs, delta=0):
    out = ""
    for symbol, (fileid, section, addr) in xrefs.items():
        if fileid < 2:
            addr += delta
        out += "%s = 0x%x ;\n" % (symbol, finallocs[(fileid, section)] + addr)
    return out

# Write LD script includes for the given sections using relative offsets
def outRelSections(locs, startsym):
    out = ""
    for addr, sectioninfo in locs:
        size, align, name = sectioninfo
        out += ". = ( 0x%x - %s ) ;\n" % (addr, startsym)
        if name == '.rodata.str1.1':
            out += "_rodata = . ;\n"
        out += "*(%s)\n" % (name,)
    return out

# Layout the 32bit segmented code.  This places the code as high as possible.
def writeLinkerScripts(locs16, locs32seg, locs32flat
                       , xref16, xref32seg, xref32flat
                       , out16, out32seg, out32flat):
    # Index to final location for each section
    # finallocs[(fileid, section)] = addr
    finallocs = {}
    for fileid, locs in ((0, locs16), (1, locs32seg), (2, locs32flat)):
        for addr, sectioninfo in locs:
            finallocs[(fileid, sectioninfo[2])] = addr

    # Write 16bit linker script
    code16_start = locs16[0][0]
    output = open(out16, 'wb')
    output.write(COMMONHEADER + outXRefs(xref16, finallocs) + """
    code16_start = 0x%x ;
    .text16 code16_start : {
""" % (code16_start)
                 + outRelSections(locs16, 'code16_start')
                 + """
    }
"""
                 + COMMONTRAILER)
    output.close()

    # Write 32seg linker script
    code32seg_start = code16_start
    if locs32seg:
        code32seg_start = locs32seg[0][0]
    output = open(out32seg, 'wb')
    output.write(COMMONHEADER + outXRefs(xref32seg, finallocs) + """
    code32seg_start = 0x%x ;
    .text32seg code32seg_start : {
""" % (code32seg_start)
                 + outRelSections(locs32seg, 'code32seg_start')
                 + """
    }
"""
                 + COMMONTRAILER)
    output.close()

    # Write 32flat linker script
    output = open(out32flat, 'wb')
    output.write(COMMONHEADER
                 + outXRefs(xref32flat, finallocs, BUILD_BIOS_ADDR) + """
    code32flat_start = 0x%x ;
    .text code32flat_start : {
""" % (locs32flat[0][0])
                 + outRelSections(locs32flat, 'code32flat_start')
                 + """
        . = ( 0x%x - code32flat_start ) ;
        *(.text32seg)
        . = ( 0x%x - code32flat_start ) ;
        *(.text16)
        code32flat_end = ABSOLUTE(.) ;
    } :text
""" % (code32seg_start + BUILD_BIOS_ADDR, code16_start + BUILD_BIOS_ADDR)
                 + COMMONTRAILER
                 + """
ENTRY(post32)
PHDRS
{
        text PT_LOAD AT ( code32flat_start ) ;
}
""")
    output.close()


######################################################################
# Section garbage collection
######################################################################

# Find and keep the section associated with a symbol (if available).
def keepsymbol(symbol, infos, pos, callerpos=None):
    addr, section = infos[pos][1].get(symbol, (None, None))
    if section is None or '*' in section or section[:9] == '.discard.':
        return -1
    if callerpos is not None and symbol not in infos[callerpos][4]:
        # This symbol reference is a cross section reference (an xref).
        # xref[symbol] = (fileid, section, addr)
        infos[callerpos][4][symbol] = (pos, section, addr)
    keepsection(section, infos, pos)
    return 0

# Note required section, and recursively set all referenced sections
# as required.
def keepsection(name, infos, pos=0):
    if name in infos[pos][3]:
        # Already kept - nothing to do.
        return
    infos[pos][3].append(name)
    relocs = infos[pos][2].get(name)
    if relocs is None:
        return
    # Keep all sections that this section points to
    for symbol in relocs:
        ret = keepsymbol(symbol, infos, pos)
        if not ret:
            continue
        # Not in primary sections - it may be a cross 16/32 reference
        ret = keepsymbol(symbol, infos, (pos+1)%3, pos)
        if not ret:
            continue
        ret = keepsymbol(symbol, infos, (pos+2)%3, pos)
        if not ret:
            continue

# Return a list of kept sections.
def getSectionsList(sections, names):
    return [i for i in sections if i[2] in names]

# Determine which sections are actually referenced and need to be
# placed into the output file.
def gc(info16, info32seg, info32flat):
    # infos = ((sections, symbols, relocs, keep sections, xrefs), ...)
    infos = ((info16[0], info16[1], info16[2], [], {}),
             (info32seg[0], info32seg[1], info32seg[2], [], {}),
             (info32flat[0], info32flat[1], info32flat[2], [], {}))
    # Start by keeping sections that are globally visible.
    for size, align, section in info16[0]:
        if section[:11] == '.fixedaddr.' or '.export.' in section:
            keepsection(section, infos)
    keepsymbol('post32', infos, 0, 2)
    # Return sections found.
    keep16 = getSectionsList(info16[0], infos[0][3]), infos[0][4]
    keep32seg = getSectionsList(info32seg[0], infos[1][3]), infos[1][4]
    keep32flat = getSectionsList(info32flat[0], infos[2][3]), infos[2][4]
    return keep16, keep32seg, keep32flat


######################################################################
# Startup and input parsing
######################################################################

# Read in output from objdump
def parseObjDump(file):
    # sections = [(size, align, section), ...]
    sections = []
    # symbols[symbol] = (addr, section)
    symbols = {}
    # relocs[section] = [symbol, ...]
    relocs = {}

    state = None
    for line in file.readlines():
        line = line.rstrip()
        if line == 'Sections:':
            state = 'section'
            continue
        if line == 'SYMBOL TABLE:':
            state = 'symbol'
            continue
        if line[:24] == 'RELOCATION RECORDS FOR [':
            state = 'reloc'
            relocsection = line[24:-2]
            continue

        if state == 'section':
            try:
                idx, name, size, vma, lma, fileoff, align = line.split()
                if align[:3] != '2**':
                    continue
                sections.append((int(size, 16), 2**int(align[3:]), name))
            except:
                pass
            continue
        if state == 'symbol':
            try:
                section, size, symbol = line[17:].split()
                size = int(size, 16)
                addr = int(line[:8], 16)
                symbols[symbol] = addr, section
            except:
                pass
            continue
        if state == 'reloc':
            try:
                off, type, symbol = line.split()
                off = int(off, 16)
                relocs.setdefault(relocsection, []).append(symbol)
            except:
                pass
    return sections, symbols, relocs

def main():
    # Get output name
    in16, in32seg, in32flat, out16, out32seg, out32flat = sys.argv[1:]

    # Read in the objdump information
    infile16 = open(in16, 'rb')
    infile32seg = open(in32seg, 'rb')
    infile32flat = open(in32flat, 'rb')

    # infoX = (sections, symbols, relocs)
    info16 = parseObjDump(infile16)
    info32seg = parseObjDump(infile32seg)
    info32flat = parseObjDump(infile32flat)

    # Figure out which sections to keep.
    # keepX = (sections, xrefs)
    keep16, keep32seg, keep32flat = gc(info16, info32seg, info32flat)

    # Determine the final memory locations of each kept section.
    # locsX = [(addr, sectioninfo), ...]
    locs16, locs32seg, locs32flat = doLayout(
        keep16[0], keep32seg[0], keep32flat[0])

    # Write out linker script files.
    writeLinkerScripts(locs16, locs32seg, locs32flat
                       , keep16[1], keep32seg[1], keep32flat[1]
                       , out16, out32seg, out32flat)

if __name__ == '__main__':
    main()



# QEMU Monitor Protocol Python class
# 
# Copyright (C) 2009 Red Hat Inc.
#
# Authors:
#  Luiz Capitulino <lcapitulino@redhat.com>
#
# This work is licensed under the terms of the GNU GPL, version 2.  See
# the COPYING file in the top-level directory.

import socket, json

class QMPError(Exception):
    pass

class QMPConnectError(QMPError):
    pass

class QEMUMonitorProtocol:
    def connect(self):
        self.sock.connect(self.filename)
        data = self.__json_read()
        if data == None:
            raise QMPConnectError
        if not data.has_key('QMP'):
            raise QMPConnectError
        return data['QMP']['capabilities']

    def close(self):
        self.sock.close()

    def send_raw(self, line):
        self.sock.send(str(line))
        return self.__json_read()

    def send(self, cmdline):
        cmd = self.__build_cmd(cmdline)
        self.__json_send(cmd)
        resp = self.__json_read()
        if resp == None:
            return
        elif resp.has_key('error'):
            return resp['error']
        else:
            return resp['return']

    def __build_cmd(self, cmdline):
        cmdargs = cmdline.split()
        qmpcmd = { 'execute': cmdargs[0], 'arguments': {} }
        for arg in cmdargs[1:]:
            opt = arg.split('=')
            try:
                value = int(opt[1])
            except ValueError:
                value = opt[1]
            qmpcmd['arguments'][opt[0]] = value
        return qmpcmd

    def __json_send(self, cmd):
        # XXX: We have to send any additional char, otherwise
        # the Server won't read our input
        self.sock.send(json.dumps(cmd) + ' ')

    def __json_read(self):
        try:
            while True:
                line = json.loads(self.sockfile.readline())
                if not 'event' in line:
                    return line
        except ValueError:
            return

    def __init__(self, filename):
        self.filename = filename
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.sockfile = self.sock.makefile()



#!/usr/bin/env python
# Script to check a bios image and report info on it.
#
# Copyright (C) 2008  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

import sys
import layoutrom

def main():
    # Get args
    objinfo, rawfile, outfile = sys.argv[1:]

    # Read in symbols
    objinfofile = open(objinfo, 'rb')
    symbols = layoutrom.parseObjDump(objinfofile)[1]
    syms = {}
    for name, (addr, section) in symbols.items():
        syms[name] = addr

    # Read in raw file
    f = open(rawfile, 'rb')
    rawdata = f.read()
    f.close()
    datasize = len(rawdata)
    finalsize = 64*1024
    if datasize > 64*1024:
        finalsize = 128*1024

    # Sanity checks
    start = syms['code32flat_start']
    end = syms['code32flat_end']
    expend = layoutrom.BUILD_BIOS_ADDR + layoutrom.BUILD_BIOS_SIZE
    if end != expend:
        print "Error!  Code does not end at 0x%x (got 0x%x)" % (
            expend, end)
        sys.exit(1)
    if datasize > finalsize:
        print "Error!  Code is too big (0x%x vs 0x%x)" % (
            datasize, finalsize)
        sys.exit(1)
    expdatasize = end - start
    if datasize != expdatasize:
        print "Error!  Unknown extra data (0x%x vs 0x%x)" % (
            datasize, expdatasize)
        sys.exit(1)

    # Print statistics
    print "Total size: %d  Free space: %d  Percent used: %.1f%% (%dKiB rom)" % (
        datasize, finalsize - datasize
        , (datasize / float(finalsize)) * 100.0
        , finalsize / 1024)

    # Write final file
    f = open(outfile, 'wb')
    f.write(("\0" * (finalsize - datasize)) + rawdata)
    f.close()

if __name__ == '__main__':
    main()
